/*
 *  Platform.mm
 *  Allenroids
 *
 *  OS-specific functions
 *  Created by Allen Smith on 1/11/14.
 *
 */
#include "Platform.h"

#import <CoreServices/CoreServices.h>
#import <CoreFoundation/CoreFoundation.h>
#import <stdlib.h>

std::string stringFromURL(CFURLRef fileURL);

/****************************************************
 * getHighScoresPath
 *  returns the path to the High Score file
 ***************************************************/
std::string getHighScoresPath()
{
	FSRef		prefFolderRef	= {{0}};
	std::string	path;
	
	// Deprecated method. But Apple offers no alternative. This is the only way
	// to get the Preferences folder without hardcoding its name!
	FSFindFolder(kUserDomain, kPreferencesFolderType, true, &prefFolderRef);
	
	CFURLRef	prefURL		= CFURLCreateFromFSRef(kCFAllocatorDefault, &prefFolderRef);
	CFURLRef	fileURL		= CFURLCreateCopyAppendingPathComponent(NULL, prefURL, CFSTR("com.AllenSmith.AllenroidsHighScores"), FALSE);
	
	path = stringFromURL(fileURL);
	
	// Cleanup
	if(prefURL) CFRelease(prefURL);
	if(fileURL) CFRelease(fileURL);
	
	return path;
}

/****************************************************
 * getPreferencesPath
 *  returns the path to the Key Configuration file
 ***************************************************/
std::string getPreferencesPath()
{
	FSRef		prefFolderRef	= {{0}};
	std::string	path;
	
	// Deprecated method. But Apple offers no alternative. This is the only way
	// to get the Preferences folder without hardcoding its name!
	FSFindFolder(kUserDomain, kPreferencesFolderType, true, &prefFolderRef);
	
	CFURLRef	prefURL		= CFURLCreateFromFSRef(kCFAllocatorDefault, &prefFolderRef);
	CFURLRef	fileURL		= CFURLCreateCopyAppendingPathComponent(NULL, prefURL, CFSTR("com.AllenSmith.AllenroidsKeyConfiguration"), FALSE);
	
	path = stringFromURL(fileURL);
	
	// Cleanup
	if(prefURL) CFRelease(prefURL);
	if(fileURL) CFRelease(fileURL);
	
	return path;
}


/****************************************************
 * stringFromURL
 *  returns a path for a CFURL
 ***************************************************/
std::string stringFromURL(CFURLRef fileURL)
{
	CFIndex		maxBufferLength	= 0;
	char*		pathBuffer		= NULL;
	std::string	path;

	CFStringRef	fileString	= CFURLCopyFileSystemPath(fileURL, kCFURLPOSIXPathStyle);
	
	maxBufferLength	= CFStringGetMaximumSizeOfFileSystemRepresentation(fileString);
	pathBuffer		= new char[maxBufferLength];
	
	CFStringGetFileSystemRepresentation(fileString, pathBuffer, maxBufferLength);
	path = pathBuffer;
	
	// Cleanup
	if(fileString) CFRelease(fileString);
	delete[] pathBuffer;
	
	return path;
}